unit pqr_bot_host_main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, bot_register;

const
  LogBoxLinesStore: integer = 1000;

type
  TPqrBotHostWin = class(TForm)
    LogBox: TMemo;
    butLoad: TButton;
    butUnload: TButton;
    butMainLoop: TButton;
    butBeginGame: TButton;
    bitQueryVersion: TButton;
    butAddPlayer: TButton;
    butRemovePlayer: TButton;
    procedure FormCreate(Sender: TObject);
    procedure butLoadClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure butUnloadClick(Sender: TObject);
    procedure butMainLoopClick(Sender: TObject);
    procedure butBeginGameClick(Sender: TObject);
    procedure bitQueryVersionClick(Sender: TObject);
    procedure butAddPlayerClick(Sender: TObject);
    procedure butRemovePlayerClick(Sender: TObject);

  private
    home_dir: string;
    php_dir: string;
    h_bot: THandle;

    dll_mainloop: TCallProc_DLL_MainLoop;
    dll_event_begingame: TCallProc_DLL_EVENT_BeginGame;
    dll_query_version: TCallProc_DLL_QUERY_VERSION;
    dll_add_player: TCallProc_DLL_SYSTEM_AddPlayer;
    dll_remove_player: TCallProc_DLL_SYSTEM_RemovePlayer;

    procedure txt_out(txt: string); overload;
    procedure txt_out(txt: string; argv: array of const); overload;

    function check_bot: boolean;

  public
    { Public declarations }
  end;

var
  PqrBotHostWin: TPqrBotHostWin;

implementation

{$R *.dfm}

procedure TPqrBotHostWin.FormCreate(Sender: TObject);
begin
  home_dir := ExtractFilePath(ParamStr(0));
  php_dir := home_dir + 'php\';
  h_bot := 0;
end;

procedure TPqrBotHostWin.FormDestroy(Sender: TObject);
begin
//
end;

procedure TPqrBotHostWin.txt_out(txt: string);
begin
  LogBox.Lines.BeginUpdate;
  try
    LogBox.Lines.Add(txt);
    while LogBox.Lines.Count > LogBoxLinesStore do
      LogBox.Lines.Delete(0);
    LogBox.SelStart := 9999999;
    SendMessage(LogBox.Handle, EM_LINESCROLL, 0, LogBox.Lines.Count);
  finally
    LogBox.Lines.EndUpdate;
  end;
end;

procedure TPqrBotHostWin.txt_out(txt: string; argv: array of const);
begin
  txt_out(Format(txt, argv));
end;

function TPqrBotHostWin.check_bot: boolean;
begin
  result := h_bot > 0;
  if not result then
    txt_out('Library is not loaded');
end;

procedure TPqrBotHostWin.butLoadClick(Sender: TObject);
begin
  txt_out('LoadLibrary(bot.dll): ');
  if h_bot <> 0 then begin
    txt_out('  ->already loaded [%d]', [h_bot]);
  end else begin
    h_bot := LoadLibrary('bot.dll');
    txt_out('  ->%d', [h_bot]);
  end;
end;

procedure TPqrBotHostWin.butUnloadClick(Sender: TObject);
begin
  txt_out('FreeLibrary(bot.dll): ');
  FreeLibrary(h_bot);
  h_bot := 0;
  txt_out('  ->ok');
end;

procedure TPqrBotHostWin.butMainLoopClick(Sender: TObject);
const
  proc_name: string = 'DLL_MainLoop';
begin
  if not check_bot then
    exit;

  if not Assigned(dll_mainloop) then begin
    txt_out('GetProcAddress(%s)...', [proc_name]);
    @dll_mainloop := GetProcAddress(h_bot, PChar(proc_name));
    if not Assigned(dll_mainloop) then begin
      txt_out('  ->error [%d]', [GetLastError]);
      exit;
    end;
    txt_out('  ->ok')
  end;

  txt_out('Call %s()...', [proc_name]);
  dll_mainloop();
  txt_out('  ->ok');
end;

procedure TPqrBotHostWin.butBeginGameClick(Sender: TObject);
const
  proc_name = 'DLL_EVENT_BeginGame';
begin
  if not check_bot then
    exit;

  if not Assigned(dll_event_begingame) then begin
    txt_out('GetProcAddress(%s)...', [proc_name]);
    @dll_event_begingame := GetProcAddress(h_bot, PChar(proc_name));
    if not Assigned(dll_event_begingame) then begin
      txt_out('  ->error [%d]', [GetLastError]);
      exit;
    end;
    txt_out('  ->ok')
  end;

  txt_out('Call %s()...', [proc_name]);
  dll_event_begingame();
  txt_out('  ->ok');
end;

procedure TPqrBotHostWin.bitQueryVersionClick(Sender: TObject);
const
  proc_name = 'DLL_QUERY_VERSION';
var
  res: string;
begin
  if not check_bot then
    exit;

  if not Assigned(dll_query_version) then begin
    txt_out('GetProcAddress(%s)...', [proc_name]);
    @dll_query_version := GetProcAddress(h_bot, PChar(proc_name));
    if not Assigned(dll_query_version) then begin
      txt_out('  ->error [%d]', [GetLastError]);
      exit;
    end;
    txt_out('  ->ok')
  end;

  txt_out('Call %s()...', [proc_name]);
  res := dll_query_version();
  txt_out('  ->%s', [res]);
end;

procedure TPqrBotHostWin.butAddPlayerClick(Sender: TObject);
const
  proc_name = 'DLL_SYSTEM_AddPlayer';
var
  player: TPlayerEx;
begin
  if not check_bot then
    exit;

  if not Assigned(dll_add_player) then begin
    txt_out('GetProcAddress(%s)...', [proc_name]);
    @dll_add_player := GetProcAddress(h_bot, PChar(proc_name));
    if not Assigned(dll_add_player) then begin
      txt_out('  ->error [%d]', [GetLastError]);
      exit;
    end;
    txt_out('  ->ok')
  end;

  txt_out('Call %s()...', [proc_name]);
  player.dead := false;
  player.bot := true;
  player.crouch := false;
  player.balloon := false;
  player.flagcarrier := true;
  player.have_rl := false;
//      , $have_gl, $have_rg, $have_bfg, $have_sg, $have_mg, $have_sh, $have_pl, $refire, $weapchg, $weapon, $threadweapon, $dir, $gantl_state, $air, $team, $item_quad, $item_regen, $item_battle, $item_flight, $item_haste, $item_invis, $ammo_mg, $ammo_sg, $ammo_gl, $ammo_rl, $ammo_sh, $ammo_rg, $ammo_pl, $ammo_bfg, $x, $y, $cx, $cy, $fangle, $InertiaX, $InertiaY, $health, $armor, $frags, $netname, $nfkmodel,
  player.location := 'Home';
  player.dxid := 123;
  dll_add_player(player);
  txt_out('  ->ok');
end;

procedure TPqrBotHostWin.butRemovePlayerClick(Sender: TObject);
const
  proc_name = 'DLL_SYSTEM_RemovePlayer';
var
  dxid: integer;
begin
  if not check_bot then
    exit;

  if not Assigned(dll_remove_player) then begin
    txt_out('GetProcAddress(%s)...', [proc_name]);
    @dll_remove_player := GetProcAddress(h_bot, PChar(proc_name));
    if not Assigned(dll_remove_player) then begin
      txt_out('  ->error [%d]', [GetLastError]);
      exit;
    end;
    txt_out('  ->ok')
  end;

  dxid := 123;
  txt_out('Call %s(%d)...', [proc_name, dxid]);
  dll_remove_player(dxid);
  txt_out('  ->ok');
end;

end.
